/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "config.h"

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

/* Gettext support */
#include <locale.h>
#include <libintl.h>

/* GUI */
#include <libappindicator/app-indicator.h>
#include <lw-gui-common.h>

static void load_plugins (GtkMenuShell *menu, GSettings *settings);

static void
load_plugins (GtkMenuShell *menu, GSettings *settings)
{
    const gchar **schemas = (const gchar **) g_settings_list_schemas();
    GSList *group = NULL;
	gint i;
	
	/* For each GSettings schemas installed on the system,
	 * if the schema has the prefix of a livewallpaper's plugin,
	 * add a radiobutton */
    for (i=0; schemas[i]; i++)
		if ( g_str_has_prefix (schemas[i], GSETTINGS_PREFIX"."))
		{
			/* label : first letter of the name in uppercase,
			 * 		   followed by the rest of the name without changes */
			GtkWidget *ctl = gtk_radio_menu_item_new_with_label (group,
								_( g_strdup_printf("%c%s",
									toupper(schemas[i][GSETTINGS_PREFIX_LEN]),
									schemas[i]+GSETTINGS_PREFIX_LEN+1)));
			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (ctl));
									
			/* Add a key "plugin" to the widget to get the string to set
			 * to the active-plugin property when the widget is activated,
			 * and connect the signal "toggled" to be notified or any changes */
			g_object_set_data (G_OBJECT (ctl), "plugin",  g_strdup (schemas[i]+GSETTINGS_PREFIX_LEN));
			g_signal_connect ( G_OBJECT (ctl), "toggled", (GCallback)_active_plugin_set, settings);

			/* Add the widget to the indicator's menu */
			gtk_menu_shell_append (menu, ctl);
		}
	/* connect the "changed" signal for key active-plugin in
	 * for settings to be notified of any change of this key,
	 * and initialize the current value of the GtkRadioMenuItem group */
	g_signal_connect (settings, "changed::active-plugin", (GCallback)_active_plugin_get, group);
	_active_plugin_get (settings, NULL, group);
}

#define SEPARATOR()	gtk_menu_shell_append(menu,gtk_separator_menu_item_new())
gint
main (gint argc, gchar **argv)
{
	GSettings *settings;
    AppIndicator *indicator;
    GtkMenuShell *menu;
    GtkWidget *ctl;
	
    /* Preparing Gettext */
    setlocale(LC_ALL, "");
    textdomain(TEXT_DOMAIN);
    bindtextdomain(TEXT_DOMAIN, DATADIR"/share/locale");

    gtk_init (&argc, &argv);

	settings = g_settings_new (GSETTINGS_PREFIX);
    indicator = app_indicator_new ( "LiveWallpaper", "livewallpaper-indicator",
									APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
    app_indicator_set_status (indicator, APP_INDICATOR_STATUS_ACTIVE);

    menu = GTK_MENU_SHELL (gtk_menu_new ());

    /* Live Wallpaper checkbutton */
    ctl = gtk_check_menu_item_new_with_label (_("LiveWallpaper"));
    g_settings_bind (settings, "active", ctl, "active", G_SETTINGS_BIND_DEFAULT);
	g_signal_connect (G_OBJECT(ctl), "activate", G_CALLBACK (livewallpaper_button_activated), settings);
    gtk_menu_shell_append (menu, ctl);

	g_bus_watch_name (G_BUS_TYPE_SESSION, "net.launchpad.Livewallpaper", G_BUS_NAME_WATCHER_FLAGS_NONE,
				      (GBusNameAppearedCallback) core_appeared, (GBusNameVanishedCallback) core_vanished,
				      ctl, NULL);

    SEPARATOR();
    load_plugins(menu, settings);
    SEPARATOR();

    /* Add a link to the configuration GUI */
    ctl = gtk_menu_item_new_with_label (_("LiveWallpaper Settings..."));
        g_signal_connect (G_OBJECT(ctl), "activate", G_CALLBACK (launch_app), "livewallpaper-config");
        gtk_menu_shell_append (menu, ctl);

    app_indicator_set_menu (indicator, GTK_MENU(menu));
    gtk_widget_show_all(GTK_WIDGET(menu));
    gtk_main ();

    return EXIT_SUCCESS;
}
